{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "8a99860f",
   "metadata": {},
   "source": [
    "# Pilas \n",
    "\n",
    "```{admonition} Objetivos\n",
    "* Objetivo 1...\n",
    "* Objetivo 2...\n",
    "```\n",
    "\n",
    "## 1. Introduccion\n",
    "\n",
    "En los programas tipicos, la cantidad de memoria que usan es desconocida. xx\n",
    "\n",
    "\n",
    "## 2. Pila (Stack) \n",
    "\n",
    "Una **pila** (**stack**) es una ADT que permite almacenar y recuperar datos. Esta presenta un modo de acceso a sus elementos de tipo **LIFO** (del inglés **Last In**, **First Out**, último en entrar, primero en salir) debido a que los datos almacenados en ella se retiran en orden inverso al que fueron entrados.\n",
    "\n",
    "```{figure} ./local/img/CH_02-S06-stack_fig1.png\n",
    "---\n",
    "name: stack_fig1\n",
    "---\n",
    "Representación y operaciones de una pila (stack)\n",
    "```\n",
    "\n",
    "Cuando se trabajan con pilas se manejan las siguientes operaciones básicas:\n",
    "* **Verificar lista vacia**: Inicia una pila vacia.\n",
    "* **Inicializar**: Inicia una pila vacia.\n",
    "* **Apilar (`push`)**: Consiste en insertar un registro al principio de una lista ligada.\n",
    "* **Desapilar (`pop`)**: Consiste en eliminar el primer nodo de la lista y retornar el dato que se hallaba en ese nodo.\n",
    "\n",
    "```{admonition} Simulación\n",
    ":class: tip\n",
    "En el siguiente [link](https://liveexample.pearsoncmg.com/dsanimation13ejava/StackeBook.html) se muestra una simulación de una pila. \n",
    "```\n",
    "\n",
    "### 2.1. Definición de la pila\n",
    "\n",
    "La siguiente estructura se define una pila (`stack`) de tamaño fijo (`CAPACITY`) cuyos miembros son:\n",
    "* **`count`**: Catidad de elementos agregados a la pila.\n",
    "* **`data`**: Arreglo donde se almacenan los elementos de la pila.\n",
    "\n",
    "```{code-block} c\n",
    "#define CAPACITY 10\n",
    "\n",
    "typedef struct _stack {\n",
    "  int count;\n",
    "  int data[CAPACITY];\n",
    "} stack;\n",
    "```\n",
    "\n",
    "La siguiente figura muestra la estructura asociada a la pila (`stack`) previamente definida:\n",
    "\n",
    "```{figure} ./local/img/CH_02-S06-stack_fig1a_.png\n",
    "---\n",
    "name: stack_fig1a\n",
    "---\n",
    "Estructura asociada a la pila (`stack`)\n",
    "```\n",
    "\n",
    "Es importante tener en cuenta, que en este caso, como la pila implementada tiene un tamaño fijo, la forma como se maneja la memoria no es la mas eficiente. Al respecto, si el tamaño (`CAPACITY`) de la pila es mucho mas grande que la cantidad de elementos almacenados, se puede estar desperdiciando una gran cantidad de memoria debido a la gran cantidad de espacio sin usar; por contrario, si el tamaño de la pila es muy pequeño, este puede hacer se la pila se quede corta cuando la cantidad de datos a manejar es mayor. \n",
    "\n",
    "```{figure} ./local/img/CH_02-S06-stack_fig1a.png\n",
    "---\n",
    "name: stack_fig1a\n",
    "---\n",
    "Comparación uso de memoria. \n",
    "```\n",
    "\n",
    "### 2.2. Funcionas de la pila\n",
    "\n",
    "Las operaciones asociadas la ADT `stack` definida previamente se destriben a continuación:\n",
    "* **`stack_init`**: crea una pila vacía:\n",
    "  \n",
    "  ```{code-block} c\n",
    "  void stack_init(stack *s) { \n",
    "    s->count = 0;\n",
    "  }\n",
    "  ```\n",
    "\n",
    "* **`stack_push`**: Agrega dato `item` a en la pila.\n",
    "\n",
    "  ```{code-block} c\n",
    "  void stack_push(stack *s, int item) {\n",
    "    assert(s != NULL);\n",
    "    assert(s->count < CAPACITY);\n",
    "    s->data[s->count] = item;\n",
    "    s->count++;\n",
    "  }\n",
    "  ```\n",
    "* **`stack_pop`**: Elimina el último elemento agregado a la pila y dejandola con un elemento menos (elementos previamente agregados).\n",
    "\n",
    "  ```{code-block} c\n",
    "  int stack_pop(stack *s) {\n",
    "    assert(s != NULL);\n",
    "    assert(s->count > 0);\n",
    "    s->count--;\n",
    "    return s->data[s->count];\n",
    "  }\n",
    "  ```\n",
    "\n",
    "* **`stack_empty`**: Retorna si la pila esta vacia `1` o `0` en caso contrario.\n",
    "  \n",
    "  ```{code-block} c\n",
    "  int stack_empty(stack *s) {\n",
    "    assert(s != NULL);\n",
    "    return (s->count == 0);\n",
    "  }\n",
    "  ```\n",
    "\n",
    "* **`stack_peek`**: Retorna el dato que está de último en la pila (el último agregado), sin eliminarlo.\n",
    "\n",
    "  ```{code-block} c\n",
    "  int stack_peek(stack *s) {\n",
    "    int top;\n",
    "    top = s->data[s->count];\n",
    "    return top;\n",
    "  }\n",
    "  ```\n",
    "\n",
    "* **`stack_full`**: Retorna si la pila esta llena `1` o `0` en caso contrario.\n",
    "\n",
    "  ```{code-block} c\n",
    "  int stack_full(stack *s) {\n",
    "    assert(s != NULL);\n",
    "    return (s->count == CAPACITY);\n",
    "  }\n",
    "  ```\n",
    "\n",
    "## 3. Uso de la estructura `stack`\n",
    "\n",
    "Recordemos que el **programa cliente** es aquel que hace el llamado de las funciones definidas en bibliotecas, modulos o clases para realizar determinadas tareas. \n",
    "\n",
    "En un programa, la función encargada de iniciar la ejecución de este es la función `main`. En esta función, se implementa la logica de la aplicación mediante el llamado a las funciones que han sido previamente definidas tal y como se muestra en la siguiente figura:\n",
    "\n",
    "\n",
    "```{figure} ./local/img/CH_02-S06-stack_fig1b.png\n",
    "---\n",
    "name: stack_fig1a\n",
    "---\n",
    "Función `main`\n",
    "```\n",
    "\n",
    "El siguiente código, muestra el caso como seria un programa de un solo archivo en el cual se hace uso de la estructura `stack`. Notese que al implementarse la logica de la aplicación función `main`, las instrucciones se definen de acuerdo a los requerimientos de la aplicación.\n",
    "\n",
    "```{code-block} c \n",
    "#include <stdio.h>\n",
    "#include <stdlib.h>\n",
    "#include <assert.h>\n",
    "\n",
    "#define CAPACITY 10\n",
    "\n",
    "/*\n",
    "Definicion de la pila\n",
    "*/\n",
    "typedef struct _stack {\n",
    "  int count;\n",
    "  int data[CAPACITY];\n",
    "} stack;\n",
    "\n",
    "\n",
    "/*\n",
    "Declaración de las funciones asociadas a la pila\n",
    "*/\n",
    "void stack_init(stack *s);\n",
    "int stack_empty(stack *s);\n",
    "void stack_push(stack *s, int item);\n",
    "int stack_pop(stack *s);\n",
    "int stack_full(stack *s);\n",
    "int stack_peek(stack *s);\n",
    "\n",
    "/*\n",
    "Funcion principal\n",
    "*/\n",
    "int main(int argc, char *argv[]) {\n",
    "  // Codigo de la aplicacion\n",
    "  ...\n",
    "  \n",
    "  return 0;\n",
    "}\n",
    "\n",
    "/*\n",
    "Definicion de las funciones asociadas a la pila\n",
    "*/\n",
    "void stack_init(stack *s) { \n",
    "  s->count = 0;\n",
    "}\n",
    "\n",
    "int stack_empty(stack *s) {\n",
    "  assert(s != NULL);\n",
    "  return (s->count == 0);\n",
    "}\n",
    "\n",
    "void stack_push(stack *s, int item) {\n",
    "  assert(s != NULL);\n",
    "  assert(s->count < CAPACITY);\n",
    "  s->data[s->count] = item;\n",
    "  s->count++;\n",
    "}\n",
    "\n",
    "int stack_pop(stack *s) {\n",
    "  assert(s != NULL);\n",
    "  assert(s->count > 0);\n",
    "  s->count--;\n",
    "  return s->data[s->count];\n",
    "}\n",
    "\n",
    "int stack_full(stack *s) {\n",
    "  assert(s != NULL);\n",
    "  return (s->count == CAPACITY);\n",
    "}\n",
    "\n",
    "int stack_peek(stack *s) {\n",
    "  assert(s != NULL);\n",
    "  int top = s->data[s->count - 1];\n",
    "  return top;\n",
    "}\n",
    "```\n",
    "\n",
    "### 3.1. Ejemplos\n",
    "\n",
    "Asumiendo una pila de 3 elementos (`CAPACITY = 3`). \n",
    "\n",
    "1. **Codigo 1**: Mediante código implemente la siguiente pila:\n",
    "   \n",
    "   ```{figure} ./local/img/CH_02-S06-stack_fig2.png\n",
    "   ---\n",
    "   name: stack_fig2\n",
    "   ---\n",
    "   Pila asociada al ejemplo.\n",
    "   ```\n",
    "      \n",
    "   El siguiente fragmento de código hace uso de las funciones definidas anteriormente para:\n",
    "   * Inicializar una pila (`S`).\n",
    "   * Agregar tres elementos a la pila: `1`, `2` y `3` respectivamente.\n",
    "   * Mostrar si la pila esta vacia, llena o parcialmente llena (indicando el numero de elementos que puede se pueden colocar).\n",
    "   * Mostrar cada vez que se agrega un elemento el valor del tope (`top`) de la pila.\n",
    "   \n",
    "   La implementación de este código se muestra a continuación ([simulación](https://pythontutor.com/render.html#code=%23include%20%3Cstdio.h%3E%0A%23include%20%3Cstdlib.h%3E%0A%23include%20%3Cassert.h%3E%0A%0A%23define%20CAPACITY%203%0A%0Atypedef%20struct%20_stack%20%7B%0A%20%20int%20count%3B%0A%20%20int%20data%5BCAPACITY%5D%3B%0A%7D%20stack%3B%0A%0Avoid%20stack_init%28stack%20*s%29%3B%0Aint%20stack_empty%28stack%20*s%29%3B%0Avoid%20stack_push%28stack%20*s,%20int%20item%29%3B%0Aint%20stack_pop%28stack%20*s%29%3B%0Aint%20stack_full%28stack%20*s%29%3B%0Aint%20stack_peek%28stack%20*s%29%3B%0A%0Aint%20main%28%29%20%7B%0A%20%20stack%20S%3B%0A%20%20stack_init%28%26S%29%3B%0A%20%20if%28stack_empty%28%26S%29%20%3D%3D%201%29%20%7B%0A%20%20%20%20printf%28%22La%20pila%20esta%20vacia%5Cn%22%29%3B%0A%20%20%7D%0A%20%20else%20if%20%28stack_full%28%26S%29%20%3D%3D%201%29%20%7B%0A%20%20%20%20printf%28%22La%20pila%20esta%20llena%5Cn%22%29%3B%0A%20%20%7D%0A%20%20else%20%7B%0A%20%20%20%20printf%28%22En%20la%20pila%20aun%20caben%20%25d%20elementos%5Cn%22,CAPACITY%20-%20S.count%29%3B%0A%20%20%7D%0A%20%20stack_push%28%26S,%201%29%3B%0A%20%20printf%28%22Elemento%20agregado%3A%20%25d%5Cn%22,%20stack_peek%28%26S%29%29%3B%0A%20%20stack_push%28%26S,%202%29%3B%0A%20%20printf%28%22Elemento%20agregado%3A%20%25d%5Cn%22,%20stack_peek%28%26S%29%29%3B%0A%20%20if%28stack_empty%28%26S%29%20%3D%3D%201%29%20%7B%0A%20%20%20%20printf%28%22La%20pila%20esta%20vacia%5Cn%22%29%3B%0A%20%20%7D%0A%20%20else%20if%20%28stack_full%28%26S%29%20%3D%3D%201%29%20%7B%0A%20%20%20%20printf%28%22La%20pila%20esta%20llena%5Cn%22%29%3B%0A%20%20%7D%0A%20%20else%20%7B%0A%20%20%20%20printf%28%22En%20la%20pila%20aun%20caben%20%25d%20elementos%5Cn%22,CAPACITY%20-%20S.count%29%3B%0A%20%20%7D%0A%20%20stack_push%28%26S,%203%29%3B%0A%20%20printf%28%22Elemento%20agregado%3A%20%25d%5Cn%22,%20stack_peek%28%26S%29%29%3B%0A%20%20if%28stack_empty%28%26S%29%20%3D%3D%201%29%20%7B%0A%20%20%20%20printf%28%22La%20pila%20esta%20vacia%5Cn%22%29%3B%0A%20%20%7D%0A%20%20else%20if%20%28stack_full%28%26S%29%20%3D%3D%201%29%20%7B%0A%20%20%20%20printf%28%22La%20pila%20esta%20llena%5Cn%22%29%3B%0A%20%20%7D%0A%20%20else%20%7B%0A%20%20%20%20printf%28%22En%20la%20pila%20aun%20caben%20%25d%20elementos%5Cn%22,CAPACITY%20-%20S.count%29%3B%0A%20%20%7D%0A%20%20return%200%3B%0A%7D%0A%0Avoid%20stack_init%28stack%20*s%29%20%7B%20%0A%20%20s-%3Ecount%20%3D%200%3B%0A%7D%0A%0Aint%20stack_empty%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20return%20%28s-%3Ecount%20%3D%3D%200%29%3B%0A%7D%0A%0Avoid%20stack_push%28stack%20*s,%20int%20item%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20assert%28s-%3Ecount%20%3C%20CAPACITY%29%3B%0A%20%20s-%3Edata%5Bs-%3Ecount%5D%20%3D%20item%3B%0A%20%20s-%3Ecount%2B%2B%3B%0A%7D%0A%0Aint%20stack_pop%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20assert%28s-%3Ecount%20%3E%200%29%3B%0A%20%20s-%3Ecount--%3B%0A%20%20return%20s-%3Edata%5Bs-%3Ecount%5D%3B%0A%7D%0A%0Aint%20stack_full%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20return%20%28s-%3Ecount%20%3D%3D%20CAPACITY%29%3B%0A%7D%0A%0Aint%20stack_peek%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20int%20top%20%3D%20s-%3Edata%5Bs-%3Ecount%20-%201%5D%3B%0A%20%20return%20top%3B%0A%7D&cumulative=false&curInstr=0&heapPrimitives=nevernest&mode=display&origin=opt-frontend.js&py=c_gcc9.3.0&rawInputLstJSON=%5B%5D&textReferences=false)):   \n",
    "   \n",
    "   ```{code-block} c\n",
    "   #include <stdio.h>\n",
    "   #include <stdlib.h>\n",
    "   #include <assert.h>\n",
    "\n",
    "   #define CAPACITY 3\n",
    "\n",
    "   typedef struct _stack {\n",
    "     int count;\n",
    "     int data[CAPACITY];\n",
    "   } stack;\n",
    "\n",
    "   // Declaración de funciones\n",
    "   // Codigo declaración...\n",
    "\n",
    "   \n",
    "   // Codigo main\n",
    "   int main() {\n",
    "     stack S;\n",
    "     stack_init(&S);\n",
    "     if(stack_empty(&S) == 1) {\n",
    "       printf(\"La pila esta vacia\\n\");\n",
    "     }\n",
    "     else if (stack_full(&S) == 1) {\n",
    "       printf(\"La pila esta llena\\n\");\n",
    "     }\n",
    "     else {\n",
    "       printf(\"En la pila aun caben %d elementos\\n\",CAPACITY - S.count);\n",
    "     }\n",
    "     stack_push(&S, 1);\n",
    "     printf(\"Elemento agregado: %d\\n\", stack_peek(&S));\n",
    "     stack_push(&S, 2);\n",
    "     printf(\"Elemento agregado: %d\\n\", stack_peek(&S));\n",
    "     if(stack_empty(&S) == 1) {\n",
    "       printf(\"La pila esta vacia\\n\");\n",
    "     }\n",
    "     else if (stack_full(&S) == 1) {\n",
    "       printf(\"La pila esta llena\\n\");\n",
    "     }\n",
    "     else {\n",
    "       printf(\"En la pila aun caben %d elementos\\n\",CAPACITY - S.count);\n",
    "     }\n",
    "     stack_push(&S, 3);\n",
    "     printf(\"Elemento agregado: %d\\n\", stack_peek(&S));\n",
    "     if(stack_empty(&S) == 1) {\n",
    "       printf(\"La pila esta vacia\\n\");\n",
    "     }\n",
    "     else if (stack_full(&S) == 1) {\n",
    "       printf(\"La pila esta llena\\n\");\n",
    "     }\n",
    "     else {\n",
    "       printf(\"En la pila aun caben %d elementos\\n\",CAPACITY - S.count);\n",
    "     }\n",
    "     return 0;\n",
    "   }\n",
    "\n",
    "   // Definición de funciones\n",
    "   // Codigo definición...\n",
    "   ```\n",
    "\n",
    "   El resultado del código anterior, se muestra a continuación:\n",
    "\n",
    "   ```{figure} ./local/img/CH_02-S06-stack_fig2a.png\n",
    "   ---\n",
    "   name: stack_fig2a\n",
    "   ---\n",
    "   Resultado código anterior\n",
    "   ```\n",
    "\n",
    "2. **Codigo 2**: Partiendo de la lista de 3 elementos anteriormente mostrada, saque el ultimo elemento agregado tal y como se muestra en la siguiente figura:\n",
    "   \n",
    "   ```{figure} ./local/img/CH_02-S06-stack_fig3.png\n",
    "   ---\n",
    "   name: stack_fig3\n",
    "   ---\n",
    "   Pila asociada al ejemplo anterior\n",
    "   ```\n",
    "   \n",
    "   La implementación realizada en el siguiente fragmento de código ([simulacion](https://pythontutor.com/render.html#code=%23include%20%3Cstdio.h%3E%0A%23include%20%3Cstdlib.h%3E%0A%23include%20%3Cassert.h%3E%0A%0A%23define%20CAPACITY%203%0A%0Atypedef%20struct%20_stack%20%7B%0A%20%20int%20count%3B%0A%20%20int%20data%5BCAPACITY%5D%3B%0A%7D%20stack%3B%0A%0Avoid%20stack_init%28stack%20*s%29%3B%0Aint%20stack_empty%28stack%20*s%29%3B%0Avoid%20stack_push%28stack%20*s,%20int%20item%29%3B%0Aint%20stack_pop%28stack%20*s%29%3B%0Aint%20stack_full%28stack%20*s%29%3B%0Aint%20stack_peek%28stack%20*s%29%3B%0A%0Aint%20main%28%29%20%7B%0A%20%20stack%20S%3B%0A%20%20stack_init%28%26S%29%3B%20%20%20%20//%20S%20%3D%20%5B%20%5D%20%3A%20Lista%20vacia%0A%20%20stack_push%28%26S,%201%29%3B%20//%20S%20%3D%20%5B%7Ctop%7C-%3E%201%5D%0A%20%20stack_push%28%26S,%202%29%3B%20//%20S%20%3D%20%5B%7Ctop%7C-%3E%202%20,%201%5D%0A%20%20stack_push%28%26S,%203%29%3B%20//%20S%20%3D%20%5B%7Ctop%7C-%3E%203,%202%20,%201%5D%0A%20%20int%20e%20%3D%20stack_pop%28%26S%29%3B%20//%20S%20%3D%20%5B%7Ctop%7C-%3E%202%20,%201%5D%20%3B%20e%20%3D%203%0A%20%20printf%28%22Elemento%20sacado%20de%20la%20pila%3A%20%25d%5Cn%22,%20e%29%3B%0A%20%20printf%28%22Elementos%20disponibles%20en%20la%20pila%3A%20%25d%20%5Cn%22,%20%20S.count%29%3B%0A%20%20return%200%3B%0A%7D%0A%0Avoid%20stack_init%28stack%20*s%29%20%7B%20%0A%20%20s-%3Ecount%20%3D%200%3B%0A%7D%0A%0Aint%20stack_empty%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20return%20%28s-%3Ecount%20%3D%3D%200%29%3B%0A%7D%0A%0Avoid%20stack_push%28stack%20*s,%20int%20item%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20assert%28s-%3Ecount%20%3C%20CAPACITY%29%3B%0A%20%20s-%3Edata%5Bs-%3Ecount%5D%20%3D%20item%3B%0A%20%20s-%3Ecount%2B%2B%3B%0A%7D%0A%0Aint%20stack_pop%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20assert%28s-%3Ecount%20%3E%200%29%3B%0A%20%20s-%3Ecount--%3B%0A%20%20return%20s-%3Edata%5Bs-%3Ecount%5D%3B%0A%7D%0A%0Aint%20stack_full%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20return%20%28s-%3Ecount%20%3D%3D%20CAPACITY%29%3B%0A%7D%0A%0Aint%20stack_peek%28stack%20*s%29%20%7B%0A%20%20assert%28s%20!%3D%20NULL%29%3B%0A%20%20int%20top%20%3D%20s-%3Edata%5Bs-%3Ecount%20-%201%5D%3B%0A%20%20return%20top%3B%0A%7D&cumulative=false&curInstr=0&heapPrimitives=nevernest&mode=display&origin=opt-frontend.js&py=c_gcc9.3.0&rawInputLstJSON=%5B%5D&textReferences=false)) hace lo siguiente:\n",
    "   * Inicializa la pila al estado inicial (`1`, `2` y `3`).\n",
    "   * Saca el ultimo elemento ingresado a la pila (`3`) imprimiendo su valor.\n",
    "   * Muestra la cantidad de elementos que quedan en la pila.\n",
    "   \n",
    "   ```{code-block} c\n",
    "   #include <stdio.h>\n",
    "   #include <stdlib.h>\n",
    "   #include <assert.h>\n",
    "\n",
    "   #define CAPACITY 3\n",
    "\n",
    "   typedef struct _stack {\n",
    "     int count;\n",
    "     int data[CAPACITY];\n",
    "   } stack;\n",
    "\n",
    "   // Declaración de funciones\n",
    "   // Codigo declaración...\n",
    "\n",
    "   \n",
    "   // Codigo main\n",
    "   int main() {\n",
    "     stack S;\n",
    "     stack_init(&S);    // S = [ ] : Lista vacia\n",
    "     stack_push(&S, 1); // S = [|top|-> 1]\n",
    "     stack_push(&S, 2); // S = [|top|-> 2 , 1]\n",
    "     stack_push(&S, 3); // S = [|top|-> 3, 2 , 1]\n",
    "     int e = stack_pop(&S); // S = [|top|-> 2 , 1] ; e = 3\n",
    "     printf(\"Elemento sacado de la pila: %d\\n\", e);\n",
    "     printf(\"Elementos disponibles en la pila: %d \\n\",  S.count);\n",
    "     return 0;\n",
    "   }\n",
    "\n",
    "   // Definición de funciones\n",
    "   // Codigo definición...\n",
    "   ```\n",
    "   \n",
    "   La salida del fragmento de código anterior, se muestra a continuación:\n",
    "\n",
    "   ```{figure} ./local/img/CH_02-S06-stack_fig3a.png\n",
    "   ---\n",
    "   name: stack_fig3\n",
    "   ---\n",
    "   Resultado código anterior\n",
    "   ```\n",
    "   \n",
    "## 4. Enlaces\n",
    "\n",
    "* https://ranger.uta.edu/~alex/courses/3318/\n",
    "* https://www.cs.usfca.edu/~galles/visualization/Algorithms.html\n",
    "* http://cslibrary.stanford.edu/\n",
    "* https://web.stanford.edu/dept/cs_edu/resources/textbook/\n",
    "* https://web.stanford.edu/class/cs106x/handouts.html\n",
    "* https://web.stanford.edu/class/cs107/\n",
    "* https://web.stanford.edu/class/archive/cs/cs107/cs107.1248/calendar\n",
    "* https://web.stanford.edu/class/cs106x/res/reader/CS106BX-Reader.pdf\n",
    "* http://cslibrary.stanford.edu/\n",
    "* https://www.cs.swarthmore.edu/~newhall/cs45/s14/#schedule\n",
    "* https://www.cs.swarthmore.edu/~newhall/unixlinks.html#lang\n",
    "* https://publications.gbdirect.co.uk/c_book/\n",
    "* https://www.cs.swarthmore.edu/~newhall/unixlinks.html#Clang\n",
    "* https://www.cs.swarthmore.edu/~newhall/unixhelp/os_stats.php\n",
    "* https://www.cs.swarthmore.edu/~newhall/cs35/\n",
    "* https://www.cs.swarthmore.edu/~newhall/unixhelp/C_linkedlists.pdf\n",
    "* https://www.cs.princeton.edu/courses/archive/spring20/cos217/ \n",
    "* https://www.cs.princeton.edu/courses/archive/spring20/cos217/precepts/09voidptrs/symtablelist.pdf\n",
    "* https://www.cs.princeton.edu/courses/archive/spring20/cos217/lectures/01_Intro.pdf\n",
    "* https://august.princeton.edu/\n",
    "* https://www.cs.princeton.edu/courses/archive/fall07/cos217/lectures/\n",
    "* https://ocw.mit.edu/courses/6-s096-introduction-to-c-and-c-january-iap-2013/\n",
    "* https://ocw.mit.edu/courses/6-s096-introduction-to-c-and-c-january-iap-2013/pages/lectures-and-assignments/data-structures-debugging/\n",
    "* https://ocw.mit.edu/courses/6-s096-introduction-to-c-and-c-january-iap-2013/pages/lectures-and-assignments/data-structures-debugging/\n",
    "* https://ocw.mit.edu/courses/6-033-computer-system-engineering-spring-2018/\n",
    "* https://ocw.mit.edu/courses/6-828-operating-system-engineering-fall-2012/\n",
    "* https://ocw.mit.edu/courses/6-087-practical-programming-in-c-january-iap-2010/\n",
    "* https://ocw.mit.edu/courses/6-087-practical-programming-in-c-january-iap-2010/pages/lecture-notes/\n",
    "* https://ocw.mit.edu/courses/1-00-introduction-to-computers-and-engineering-problem-solving-spring-2012/\n",
    "* https://ocw.mit.edu/courses/6-0002-introduction-to-computational-thinking-and-data-science-fall-2016/\n",
    "* https://ocw.mit.edu/courses/6-100l-introduction-to-cs-and-programming-using-python-fall-2022/\n",
    "* https://ocw.mit.edu/collections/introductory-programming/\n",
    "* https://cs61c.org/su24/\n",
    "* https://www.cs.princeton.edu/courses/archive/spr24/cos126/schedule/\n",
    "* https://web2.qatar.cmu.edu/~mhhammou/15122-s23/\n",
    "* https://web2.qatar.cmu.edu/~mhhammou/15122-s23/schedule.html\n",
    "* https://bytesoftheday.wordpress.com/2014/07/04/q14/\n",
    "* https://www.cs.princeton.edu/courses/archive/spring20/cos217/\n",
    "* https://embeddedwala.com/Blogs/embedded-c/memory-layout-of-c-program\n",
    "* https://www.cs.mtsu.edu/~cs2170/C++labs/lab18/OSmemlayout.pdf\n",
    "* https://d1b10bmlvqabco.cloudfront.net/attach/j6fe5friemd22w/hzd1madqsie3ts/j7kw6i4tmqf8/61C_Note_1_Memory.pdf\n",
    "* https://ocw.mit.edu/courses/6-s096-introduction-to-c-and-c-january-iap-2013/bba9056d5290198d563edc47dfcff0e9_MIT6_S096_IAP13_lec3.pdf\n",
    "* https://cs61c.org/su24/\n",
    "* http://wla.berkeley.edu/~cs61c/fa17/\n",
    "* https://www.cs.princeton.edu/courses/archive/fall07/cos217/index.html\n",
    "* https://web2.qatar.cmu.edu/~mhhammou/15122-s23/lectures/21-cmem/writeup/pdf/main.pdf\n",
    "* https://cs.gmu.edu/~zduric/cs262/Slides/teoX.pdf\n",
    "* https://d1b10bmlvqabco.cloudfront.net/attach/j6fe5friemd22w/hzd1madqsie3ts/j7kw6i4tmqf8/61C_Note_1_Memory.pdf\n",
    "* https://www.cs.princeton.edu/courses/archive/fall07/cos217/\n",
    "* https://www.cs.mtsu.edu/~cs2170/C++labs/lab18/OSmemlayout.pdf\n",
    "* https://web2.qatar.cmu.edu/~mhhammou/15122-s23/lectures/21-cmem/writeup/pdf/main.pdf\n",
    "* https://www.cs.princeton.edu/courses/archive/spr24/cos126/schedule/\n",
    "* https://github.com/vishwa27yvs/Intro-to-Computer-Science-COS-126\n",
    "* https://www.berthon.eu/wiki/foss:wikishelf:linux:memory\n",
    "* http://resources.infosecinstitute.com/system-address-map-initialization-in-x86x64-architecture-part-1-pci-based-systems/#gref\n",
    "* https://fypandroid.wordpress.com/2011/01/17/anatomy-of-a-program-in-memory/\n",
    "* https://www.securitysift.com/windows-exploit-development-part-1-basics/\n",
    "* https://www.ibm.com/developerworks/library/j-nativememory-linux/\n",
    "* https://gabrieletolomei.wordpress.com/miscellanea/operating-systems/in-memory-layout/\n",
    "* http://www.cs.utexas.edu/users/fussell/cs310h/lectures/Lecture_17-310h.pdf\n",
    "* https://ocw.mit.edu/courses/electrical-engineering-and-computer-science/6-087-practical-programming-in-c-january-iap-2010/lecture-notes/\n",
    "* https://stackoverflow.com/questions/2128728/allocate-matrix-in-c\n",
    "* https://www.geeksforgeeks.org/dynamically-allocate-2d-array-c/\n",
    "* https://www.programiz.com/c-programming/c-dynamic-memory-allocation\n",
    "* https://www.cs.swarthmore.edu/~newhall/unixhelp/C_arrays.html"
   ]
  }
 ],
 "metadata": {
  "jupytext": {
   "cell_metadata_filter": "-all",
   "formats": "md:myst",
   "text_representation": {
    "extension": ".md",
    "format_name": "myst",
    "format_version": 0.13,
    "jupytext_version": "1.11.5"
   }
  },
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "source_map": [
   14
  ]
 },
 "nbformat": 4,
 "nbformat_minor": 5
}